/*------------------------------------------------------------------------------
 Copyright:      Markus Kotschenreuther
 Author:         Markus Kotschenreuther
 Remarks:        
 known Problems: none
 Version:        25.02.2009
 Description:    Digital IOs Abfragen / Setzen
 
 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "digital_io.h"
#if(DEBUG_LEVEL >= 2)
	#include "stdio.h"
#endif

#include "std_defs.h"
#include "lcd.h"
#include "../../projShared/lanswitch.h"
	
 // Ports Pins
#define PIN0		0
#define PIN1		1
#define PIN2		2
#define PIN3		3
#define PIN4		4
#define PIN5		5
#define PIN6		6
#define PIN7		7

//############################################################################
void DigIO_Init(void)
//############################################################################
{ 

	//Konfiguration der Ausgnge bzw. Eingnge

	// +++ Port A +++
	//**************************
	// xxxx 0000 -> PA0/3 = digIN; PA4/7 = analogIn (defined in analog.h)
	//**************************
	DDRA = 0x00;
	// Set PORT Register
	// init register first
	PORTA  = 0x00;
	// set pull ups
	PORTA |= (1<<PIN0);
	PORTA |= (1<<PIN1);
	PORTA |= (1<<PIN2);
	PORTA |= (1<<PIN3);

	// +++ Port B +++
	//**************************
	// xxxx xxxx -> SPI interface to ENC chip (ethernet phy)
	//**************************

	// +++ Port C +++
	//**************************
	// 1111 1111 -> digital outputs
	//**************************
	DDRC = 0xFF;

	// +++ Port D +++
	//**************************
	// 0x01 11xx -> PD0/1 = RS232; PD2/4 = LCDout; PD5,7 buttonIN;
	//              PD6 used as HW ICP1 interrupt Pin for Humid sensor
	//**************************
	// Set Port Direction Register
	// init everything as input
	DDRD = 0x00;

	// define outputs
	DDRD |= (1<<LCD_DATA_ENABLE);
	DDRD |= (1<<LCD_CLOCK);
	DDRD |= (1<<LCD_DATA);
	DDRD |= (1<<PIN5);
	DDRD |= (1<<PIN7);

	// Set PORT Register
	// init register first
	PORTD  = 0x00;
	// set pull ups
	PORTD |= (1<<PIN6);
}


//############################################################################
void SetChannel(unsigned short port, unsigned short pin, unsigned short value)
//############################################################################
{
	unsigned char register_status = 0;
	unsigned char bit2set = 0;

#if(DEBUG_LEVEL >= 2)
	printf("DBG:%i:%i=:%i\n", port, pin, value);
#endif

	switch (port){
		case 1:{
			register_status = PORTA;
			break;
		}
		case 2:{
			register_status = PORTB;
			break;
		}
		case 3:{
			register_status = PORTC;
			break;
		}
		case 4:{
			register_status = PORTD;
			break;
		}
	}

	// create a bitmask
	bit2set = (0x01 << pin);

	//ON
	if ((value == ON)&&((register_status & bit2set) == 0)){
		register_status = (register_status | bit2set);
	}

	//OFF
 	if ((value == OFF)&&((register_status & bit2set) != 0)){
 		register_status = (register_status ^ bit2set);
	}
	switch (port){
		case 1:{
			PORTA = register_status;
			break;
		}
		case 2:{
			PORTB = register_status;
			break;
		}
		case 3:{
			PORTC = register_status;;
			break;
		}
		case 4:{
			PORTD = register_status;
			break;
		}
	}
	return;
}

//############################################################################
unsigned int GetDirectChannel(unsigned short port, unsigned short pin)
//############################################################################
{
	char register_status = 0;
	char bit2read = 0;

	switch (port){

		case 1:{
			register_status = PINA;
			break;
		}
		case 2:{
			register_status = PINB;
			break;
		}
		case 3:{
			register_status = PINC;
			break;
		}
		case 4:{
			register_status = PIND;
			break;
		}
		default:
			return(2);
	}

	// create a bitmask
	bit2read = (0x01 << pin);

	// pull ups activated -> 0 == ON; 1 == OFF
	return ((register_status & bit2read) == 0)?ON:OFF;
}

